@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2015-2019 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2015-2020 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -130,8 +130,8 @@ package FunctionalPerl::Htmlgen::Nav::TopEntry {
         array_to_hashset);
 
     use FP::Struct [
-        [*is_procedure,                                           "nav_bar"],
-        [maybe instance_of "FunctionalPerl::Htmlgen::Nav::Index", "index"]
+        [*is_procedure,                                            "nav_bar"],
+        [maybe(instance_of "FunctionalPerl::Htmlgen::Nav::Index"), "index"]
         ],
         "FunctionalPerl::Htmlgen::Nav::Entry";
 
@@ -145,10 +145,12 @@ package FunctionalPerl::Htmlgen::Nav::TopEntry {
 
     sub item_is_in_lower_hierarchy($self) {
         compose(
-            hashset_to_predicate array_to_hashset(
-                $self->subentries_of_subentries->flatten->map(
-                    the_method "path0"
-                )->array
+            hashset_to_predicate(
+                array_to_hashset(
+                    $self->subentries_of_subentries->flatten->map(
+                        the_method "path0"
+                    )->array
+                )
             ),
             the_method "path0"
         )
