@@ -151,6 +151,7 @@ sub import { }
 use Symbol;
 use Carp;
 use Fcntl qw(:DEFAULT :flock :seek :mode);
+use FP::Carp;
 
 my $has_posix;
 
@@ -425,7 +426,7 @@ sub xread {
 
 sub xreadchunk {
     my $self = shift;
-    @_ == 1 or croak "xreadchunk: expecting 1 parameter (length)";
+    @_ == 1 or fp_croak_nargs 1;
     my $buf;
     my $rv = CORE::read $self->fh, $buf, $_[0];
     defined $rv or croak "xreadchunk " . $self->quotedname . ": $!";
@@ -434,7 +435,7 @@ sub xreadchunk {
 
 sub xxreadchunk {
     my $self = shift;
-    @_ == 1 or croak "xreadchunk: expecting 1 parameter (length)";
+    @_ == 1 or fp_croak_nargs 1;
     my $buf;
     my $rv = CORE::read $self->fh, $buf, $_[0];
     defined $rv or croak "xreadchunk " . $self->quotedname . ": $!";
@@ -991,7 +992,7 @@ sub seek {
 
 sub xtell {
     my $self = shift;
-    @_ == 0 or croak "xtell: wrong number of arguments";
+    @_ == 0 or fp_croak_nargs 0;
     my $res = tell $self->fh;
     if ($res == -1) {
         croak "xtell on " . ($self->quotedname) . ": $!";
@@ -1002,7 +1003,7 @@ sub xtell {
 
 sub tell {
     my $self = shift;
-    @_ == 0 or croak "tell: wrong number of arguments";
+    @_ == 0 or fp_croak_nargs 0;
     my $res = tell $self->fh;
     if ($res == -1) {
         undef
