@@ -81,6 +81,7 @@ our @EXPORT_OK   = qw(run_tests run_tests_ no_tests);
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
 use Carp;
+use FP::Carp;
 use Chj::singlequote;
 
 # get the style
@@ -173,7 +174,8 @@ our $num_by_package   = {};
 
 our $dropped_tests = 0;
 
-sub no_tests () {
+sub no_tests {
+    @_ == 0 or fp_croak_nargs 0;
     exists $ENV{TEST} and !$ENV{TEST}
 }
 
@@ -256,7 +258,8 @@ package Chj::TEST::Test::Builder {
 use FP::Equal qw(relaxedequal);
 use FP::Show;
 
-sub eval_test ($$) {
+sub eval_test {
+    @_ == 2 or fp_croak_nargs 2;
     my ($t, $stat) = @_;
     my ($proc, $res, $num, $package, $filename, $line) = @$t;
     style_switch {
