@@ -97,10 +97,12 @@ our @EXPORT_OK = qw(xopen_write xopen_append xopen_update
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
 use Carp;
+use FP::Carp;
 use Chj::IO::File;
 use Scalar::Util 'blessed';
 
-sub glob_to_fh ($;$) {
+sub glob_to_fh {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($glob, $maybe_layer_or_encoding) = @_;
     my $fh = bless(*{$glob}{IO}, "Chj::IO::File");
     $fh->perhaps_set_layer_or_encoding($maybe_layer_or_encoding);
@@ -115,7 +117,8 @@ sub glob_to_fh ($;$) {
 
 use IO::Handle;
 
-sub fd_to_fh ($$;$) {
+sub fd_to_fh {
+    @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
     my ($fd, $mode, $maybe_layer_or_encoding) = @_;
     $fd =~ /^\d+\z/s or die "fd argument must be a natural number";
     my $fh = IO::Handle->new_from_fd($fd, $mode);
@@ -124,17 +127,20 @@ sub fd_to_fh ($$;$) {
     $fh
 }
 
-sub inout_fd_to_fh ($;$) {
+sub inout_fd_to_fh {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($fd, $maybe_layer_or_encoding) = @_;
     fd_to_fh $fd, "rw", $maybe_layer_or_encoding
 }
 
-sub input_fd_to_fh ($;$) {
+sub input_fd_to_fh {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($fd, $maybe_layer_or_encoding) = @_;
     fd_to_fh $fd, "r", $maybe_layer_or_encoding
 }
 
-sub output_fd_to_fh ($;$) {
+sub output_fd_to_fh {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($fd, $maybe_layer_or_encoding) = @_;
     fd_to_fh $fd, "w", $maybe_layer_or_encoding
 }
@@ -143,13 +149,15 @@ sub output_fd_to_fh ($;$) {
 # Wrap a Perl fh of another kind (class) as a Chj::IO::File handle,
 # for cases where reblessing is not ok.
 
-sub fh_to_fh ($) {
+sub fh_to_fh {
+    @_ == 1 or fp_croak_nargs 1;
     my ($fh) = @_;
     require Chj::IO::WrappedFile;
     Chj::IO::WrappedFile->new($fh)
 }
 
-sub possibly_fh_to_fh ($) {
+sub possibly_fh_to_fh {
+    @_ == 1 or fp_croak_nargs 1;
     my ($fh) = @_;
     if (defined blessed($fh) and $fh->isa("Chj::IO::File")) {
         $fh
@@ -167,7 +175,8 @@ sub xopen {
         ;    # (evil?, should it use ->can to remain OO based?)
 }
 
-sub xopen_read($) {
+sub xopen_read {
+    @_ == 1 or fp_croak_nargs 1;
     if ($_[0] =~ /^((<)|( >> )|(>)|(\+<)|(\+>))/) {
         croak "xopen_read: mode $1 not allowed"
             unless $2;    # XXX isn't this wong? Too many parens above?
@@ -182,21 +191,22 @@ sub xopen_read($) {
 
 # XX ok to simply use the 3-argument open and never allow 2-open
 # strings at all? See how I seem to have gotten it wrong anyway, above!
-sub perhaps_xopen_read ($) {
-    @_ == 1 or die "wrong number of arguments";
+sub perhaps_xopen_read {
+    @_ == 1 or fp_croak_nargs 1;
     unshift @_, "<";
     unshift @_, 'Chj::IO::File';
     goto &Chj::IO::File::perhaps_xopen;
 }
 
-sub perhaps_open_read ($) {
-    @_ == 1 or die "wrong number of arguments";
+sub perhaps_open_read {
+    @_ == 1 or fp_croak_nargs 1;
     unshift @_, "<";
     unshift @_, 'Chj::IO::File';
     goto &Chj::IO::File::perhaps_open;
 }
 
-sub xopen_write($) {
+sub xopen_write {
+    @_ == 1 or fp_croak_nargs 1;
     if ($_[0] =~ /^((<)|( >> )|(>)|(\+<)|(\+>))/) {
         croak "xopen_write: mode $1 not allowed" unless $3 or $4;
     } elsif (@_ == 1 and $_[0] eq '-') {
@@ -208,7 +218,8 @@ sub xopen_write($) {
     goto &Chj::IO::File::xopen;
 }
 
-sub xopen_append($) {
+sub xopen_append {
+    @_ == 1 or fp_croak_nargs 1;
     if ($_[0] =~ /^((<)|( >> )|(>)|(\+<)|(\+>))/) {
         croak "xopen_append: mode $1 not allowed" unless $3;
     } elsif (@_ == 1 and $_[0] eq '-') {
@@ -220,7 +231,8 @@ sub xopen_append($) {
     goto &Chj::IO::File::xopen;
 }
 
-sub xopen_update($) {
+sub xopen_update {
+    @_ == 1 or fp_croak_nargs 1;
     if ($_[0] =~ /^((<)|( >> )|(>)|(\+<)|(\+>))/) {
         croak "xopen_update: mode $1 not allowed" unless $5 or $6;
     } elsif (@_ == 1 and $_[0] eq '-') {
