@@ -159,6 +159,7 @@ our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
 use FP::Lazy 'force';
 use Safe::Isa;
+use FP::Carp;
 
 package FP::Failure::Failure {
 
@@ -245,7 +246,8 @@ package FP::Failure::Failure {
 
 our $trace_failures = 0;    # bool
 
-sub failure ($;$) {
+sub failure {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($value, $maybe_parents) = @_;
     my $v = FP::Failure::Failure->new(
         $value,
@@ -267,13 +269,15 @@ sub failure ($;$) {
     defined wantarray ? $v : die $v
 }
 
-sub is_failure($) {
+sub is_failure {
+    @_ == 1 or fp_croak_nargs 1;
     force($_[0])->$_isa("FP::Failure::Failure")
 }
 
 our $use_failure = 0;    # bool
 
-sub fails ($;$) {
+sub fails {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     $use_failure            ? &failure(@_)
         : defined wantarray ? 0
         :                     die "fails called in void context";
@@ -286,12 +290,13 @@ package FP::Failure::Abstract::Message {
 
 package FP::Failure::Message {
     use FP::Show;
+    use FP::Carp;
 
     use FP::Struct ['messagestring', 'arguments'],
         'FP::Failure::Abstract::Message';
 
     sub message {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s    = shift;
         my $args = $s->arguments;
         my $msg  = $s->messagestring;
@@ -307,12 +312,13 @@ sub message {
 
 package FP::Failure::MessageFmt {
     use FP::Show;
+    use FP::Carp;
 
     use FP::Struct ['formatstring', 'arguments'],
         'FP::Failure::Abstract::Message';
 
     sub message {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         sprintf($s->formatstring, map { show $_ } @{ $s->arguments })
     }
