@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014-2015 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2014-2020 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -78,6 +78,7 @@ use Chj::xopen qw(
     glob_to_fh
 );
 use Chj::xtmpfile qw(xtmpfile);
+use FP::Carp;
 
 # XX use this for the definitions further below instead of re-coding
 # it each time?
@@ -100,7 +101,8 @@ sub maybeIO_to_stream {
     &{ Weakened $next}
 }
 
-sub _perhaps_opendir_stream ($) {
+sub _perhaps_opendir_stream {
+    @_ == 1 or fp_croak_nargs 1;
     my ($path) = @_;
     if (my ($d) = perhaps_opendir $path) {
         my $next;
@@ -121,7 +123,8 @@ sub _perhaps_opendir_stream ($) {
     }
 }
 
-sub _perhaps_opendir_stream_sorted ($$) {
+sub _perhaps_opendir_stream_sorted {
+    @_ == 2 or fp_croak_nargs 2;
     my ($path, $cmp) = @_;
     if (my ($d) = perhaps_opendir $path) {
         my $items = array_sort [$d->xnread], $cmp;
@@ -132,7 +135,8 @@ sub _perhaps_opendir_stream_sorted ($$) {
     }
 }
 
-sub perhaps_directory_items ($;$) {
+sub perhaps_directory_items {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($path, $maybe_cmp) = @_;
     if ($maybe_cmp) {
         _perhaps_opendir_stream_sorted $path, $maybe_cmp;
@@ -141,7 +145,8 @@ sub perhaps_directory_items ($;$) {
     }
 }
 
-sub perhaps_directory_paths ($;$) {
+sub perhaps_directory_paths {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($base, $maybe_cmp) = @_;
     $base .= "/" unless $base =~ /\/\z/;
     if (my ($s) = perhaps_directory_items $base, $maybe_cmp) {
@@ -156,7 +161,8 @@ sub perhaps_directory_paths ($;$) {
     }
 }
 
-sub xdirectory_items ($;$) {
+sub xdirectory_items {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($path, $maybe_cmp) = @_;
     if (my ($s) = perhaps_directory_items($path, $maybe_cmp)) {
         $s
@@ -165,7 +171,8 @@ sub xdirectory_items ($;$) {
     }
 }
 
-sub xdirectory_paths ($;$) {
+sub xdirectory_paths {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($path, $maybe_cmp) = @_;
     if (my ($s) = perhaps_directory_paths($path, $maybe_cmp)) {
         $s
@@ -174,7 +181,8 @@ sub xdirectory_paths ($;$) {
     }
 }
 
-sub fh_to_stream ($$$) {
+sub fh_to_stream {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fh, $read, $close) = @_;
     my $next;
     $next = sub {
@@ -224,7 +232,8 @@ sub xfile_lines_chomp;
 
 # Clojure calls this line-seq
 #  (http://clojure.github.io/clojure/clojure.core-api.html#clojure.core/line-seq)
-sub fh_to_lines ($) {
+sub fh_to_lines {
+    @_ == 1 or fp_croak_nargs 1;
     my ($fh) = @_;
     fh_to_stream(possibly_fh_to_fh($fh), the_method("xreadline"),
         the_method("xclose"))
@@ -235,7 +244,8 @@ sub fh_to_lines ($) {
 # bufiz (since only xsysreadcompletely would guarantee to fill size,
 # but would die on mid-chunk EOF)
 
-sub fh_to_chunks ($$) {
+sub fh_to_chunks {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fh, $bufsiz) = @_;
     fh_to_stream(
         possibly_fh_to_fh($fh),
@@ -252,7 +262,8 @@ sub fh_to_chunks ($$) {
 # when each cell is being forced. Optional argument in seconds
 # (floating point) to sleep before returning the next element.
 
-sub timestream (;$) {
+sub timestream {
+    @_ >= 0 and @_ <= 1 or fp_croak_nargs "0-1";
     my ($maybe_sleep) = @_;
     require Time::HiRes;
     my $lp;
@@ -265,10 +276,10 @@ sub timestream (;$) {
     Weakened($lp)->();
 }
 
-sub xstream_print ($;$) {
+sub xstream_print {
     @_ == 2 or @_ == 1 or die "wrong number of arguments";
     my ($s, $maybe_fh) = @_;
-    my $fh = $maybe_fh // glob_to_fh * STDOUT;
+    my $fh = $maybe_fh // glob_to_fh(*STDOUT);
     weaken $_[0];
     $s->for_each(
         sub {
@@ -277,7 +288,7 @@ sub xstream_print ($;$) {
     );
 }
 
-sub xstream_to_file ($$;$) {
+sub xstream_to_file {
     @_ == 2 or @_ == 3 or die "wrong number of arguments";
     my ($s, $path, $maybe_mode) = @_;
     my $out = xtmpfile $path;
@@ -290,7 +301,8 @@ sub xstream_to_file ($$;$) {
 # read and write back a file, passing its lines as a stream to the
 # given function; written to temp file that's renamed into place upon
 # successful completion.
-sub xfile_replace_lines ($$) {
+sub xfile_replace_lines {
+    @_ == 2 or fp_croak_nargs 2;
     my ($path, $fn) = @_;
     xstream_to_file &$fn(xfile_lines $path), $path;
 }
