@@ -196,6 +196,8 @@ use FP::Show;
 use Scalar::Util qw(weaken blessed);
 use FP::Weak qw(Weakened);
 use FP::Interfaces;
+use Carp;
+use FP::Carp;
 
 our $immutable = 1;    # whether pairs are to be made immutable
 
@@ -204,6 +206,7 @@ our $immutable = 1;    # whether pairs are to be made immutable
 
 package FP::List::List {
     use FP::Lazy;
+    use FP::Carp;
 
     use Chj::NamespaceCleanAbove;
 
@@ -211,30 +214,26 @@ package FP::List::List {
 
     # return this sequence as a list, i.e. identity
     sub list {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         $_[0]
     }
 
     sub stream {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($l) = @_;
         lazy {$l}
     }
 
     sub strictlist {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         FP::StrictList::strictlist($s->values)
     }
 
-    sub purearray {
-        @_ == 1 or die "wrong number of arguments";
-        my $s = shift;
-        FP::_::PureArray->new_from_array([$s->values])
-    }
+    # sub purearray: see below, *FP::List::List::purearray
 
     sub mutablearray {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         FP::_::MutableArray->new_from_array([$s->values])
     }
@@ -248,6 +247,7 @@ package FP::List::List {
 }
 
 package FP::List::Null {
+    use FP::Carp;
     our @ISA = qw(FP::List::List);
 
     sub pair_namespace {"FP::List::Pair"}
@@ -256,7 +256,7 @@ package FP::List::Null {
 
     sub cons {
         my $s = shift;
-        @_ == 1 or die "expecting 1 method argument";
+        @_ == 1 or fp_croak_nargs 1;
         my @p = ($_[0], $s);
         bless \@p, $s->pair_namespace;
         if ($immutable) {
@@ -303,13 +303,14 @@ package FP::List::Null {
 }
 
 package FP::List::Pair {
+    use FP::Carp;
     our @ISA = qw(FP::List::List);
 
     sub is_null {''}
 
     sub cons {
         my $s = shift;
-        @_ == 1 or die "expecting 1 method argument";
+        @_ == 1 or fp_croak_nargs 1;
         my @p = ($_[0], $s);
         bless \@p, ref($s);
         if ($immutable) {
@@ -323,33 +324,33 @@ package FP::List::Pair {
     sub car {
         $_[0][0]
     }
-    *first = *car;
+    *first = \&car;
 
-    *maybe_first   = *first;
-    *perhaps_first = *first;
+    *maybe_first   = \&first;
+    *perhaps_first = \&first;
 
-    *first_set    = *FP::List::first_set;
-    *first_update = *FP::List::first_update;
+    *first_set    = \&FP::List::first_set;
+    *first_update = \&FP::List::first_update;
 
     sub cdr {
         $_[0][1]
     }
-    *rest         = *cdr;
-    *maybe_rest   = *rest;
-    *perhaps_rest = *rest;
+    *rest         = \&cdr;
+    *maybe_rest   = \&rest;
+    *perhaps_rest = \&rest;
 
     sub car_and_cdr {
         @{ $_[0] }
     }
-    *first_and_rest         = *car_and_cdr;
-    *perhaps_first_and_rest = *car_and_cdr;
+    *first_and_rest         = \&car_and_cdr;
+    *perhaps_first_and_rest = \&car_and_cdr;
 
     sub cddr   { $_[0]->cdr->cdr }
     sub cdddr  { $_[0]->cdr->cdr->cdr }
     sub cddddr { $_[0]->cdr->cdr->cdr->cdr }
 
     sub cadr { $_[0]->cdr->car }
-    *second = *cadr;
+    *second = \&cadr;
     sub caddr   { $_[0]->cdr->cdr->car }
     sub cadddr  { $_[0]->cdr->cdr->cdr->car }
     sub caddddr { $_[0]->cdr->cdr->cdr->cdr->car }
@@ -406,8 +407,8 @@ TEST {
 }
 1;
 
-sub cons ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub cons {
+    @_ == 2 or fp_croak_nargs 2;
     if (defined blessed($_[1]) and my $f = $_[1]->can("cons")) {
         @_ = ($_[1], $_[0]);
         goto &$f;
@@ -416,11 +417,11 @@ sub cons ($$) {
     }
 }
 
-sub cons_ ($) {
-    @_ == 1 or die "wrong number of arguments";
+sub cons_ {
+    @_ == 1 or fp_croak_nargs 1;
     my ($item) = @_;
     sub {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         if (defined blessed($_[0]) and my $f = $_[0]->can("cons")) {
             push @_, $item;
             goto &$f;
@@ -433,8 +434,8 @@ sub cons_ ($) {
 TEST { cons_(1)->(list(2)) } GIVES { list(1, 2) };
 TEST { cons_(1)->(2) } GIVES       { cons 1, 2 };
 
-sub pair ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub pair {
+    @_ == 2 or fp_croak_nargs 2;
     goto \&unsafe_cons
 }
 
@@ -444,7 +445,8 @@ sub pair ($$) {
 # was confirmed to be a pair with `is_pair`. unsafe_cons is safe if
 # the rest argument should never dictate the type of the result.
 
-sub unsafe_cons ($$) {
+sub unsafe_cons {
+    @_ == 2 or fp_croak_nargs 2;
     my @p = @_;
     bless \@p, "FP::List::Pair";
     if ($immutable) {
@@ -457,18 +459,19 @@ sub unsafe_cons ($$) {
 
 # WARNING: be careful, this isn't safe even if `is_pair` returns true, as
 # that only assures that ->car etc. can be called.
-sub unsafe_car ($) {
+sub unsafe_car {
+    @_ == 1 or fp_croak_nargs 1;
     $_[0][0]
 }
 
 # WARNING: same as for unsafe_car
-sub unsafe_cdr ($) {
+sub unsafe_cdr {
+    @_ == 1 or fp_croak_nargs 1;
     $_[0][1]
 }
 
-sub is_pair ($);
-
-sub is_pair ($) {
+sub is_pair {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     (
@@ -480,16 +483,17 @@ sub is_pair ($) {
         # ^  XX evil: inlined `is_promise`
 }
 
-sub is_pair_noforce ($) {
+sub is_pair_noforce {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     ($r eq "FP::List::Pair" or $v->isa("FP::List::Pair"))
 }
 
-sub is_pair_of ($$) {
+sub is_pair_of {
     my ($p0, $p1) = @_;
     sub {
-        @_ == 1 or die "expecting 1 argument";
+        @_ == 1 or fp_croak_nargs 1;
         my ($v) = @_;
         (is_pair($v) and &$p0($$v[0]) and &$p1($$v[1]))
     }
@@ -506,15 +510,17 @@ my $null = do {
 Internals::SvREADONLY $null, 1;
 
 sub null () {
+
+    # @_ == 0 or fp_croak_nargs 0;
+    # nope, it is also called as a method
     $null
 }
 
 TEST { null->cons(1)->cons(2)->array }
 [2, 1];
 
-sub is_null ($);
-
-sub is_null ($) {
+sub is_null {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     (
@@ -524,15 +530,15 @@ sub is_null ($) {
     )
 }
 
-sub is_null_noforce ($) {
+sub is_null_noforce {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     ($r eq "FP::List::Null" or $v->isa("FP::List::Null"))
 }
 
-sub is_pair_or_null ($);
-
-sub is_pair_or_null ($) {
+sub is_pair_or_null {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     (
@@ -549,18 +555,19 @@ TEST { is_pair_or_null bless [], "NirvAna" } '';
 
 # test subclassing? whatever
 
-sub is_null_or_pair_of ($$$);
-
-sub is_null_or_pair_of ($$$) {
+sub is_null_or_pair_of {
+    @_ == 3 or fp_croak_nargs 3;
     my ($v, $p0, $p1) = @_;
     FORCE $v;
     (is_null $v or (is_pair $v and &$p0($v->car) and &$p1($v->cdr)))
 }
 
-sub null_or_pair_of ($$) {
+sub null_or_pair_of {
+    @_ == 2 or fp_croak_nargs 2;
     my ($p0, $p1) = @_;
 
-    sub ($) {
+    sub {
+        @_ == 1 or fp_croak_nargs 1;
         my ($v) = @_;
         is_null_or_pair_of($v, $p0, $p1)
     }
@@ -569,7 +576,7 @@ sub null_or_pair_of ($$) {
 TEST {
     require FP::Array;
     FP::Array::array_map(
-        null_or_pair_of(*is_null, *is_pair),
+        null_or_pair_of(\&is_null, \&is_pair),
         [
             null,
             cons(1,    2),
@@ -582,7 +589,8 @@ TEST {
 }
 [1, undef, undef, '', 1, ''];
 
-sub is_list ($) {
+sub is_list {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     FORCE $v;
     (
@@ -609,14 +617,14 @@ TEST {
 }
 '';
 
-use Carp;
-
-sub die_not_a_pair ($) {
+sub die_not_a_pair {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     croak "not a pair: " . show($v);
 }
 
-sub car ($) {
+sub car {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // die_not_a_pair($v);
     if ($r eq "FP::List::Pair") {
@@ -629,20 +637,22 @@ sub car ($) {
     }
 }
 
-sub first ($);
-*first = *car;
+sub first;
+*first = \&car;
 
 # XX add maybe_first and perhaps_first wrappers here? Shouldn't this
 # be more structured/automatic, finally.
 
-sub first_set ($$) {
+sub first_set {
+    @_ == 2 or fp_croak_nargs 2;
     my ($p, $v) = @_;
     cons($v, $p->rest)
 }
 
 TEST { cons(3, 4)->first_set("a") } bless ["a", 4], 'FP::List::Pair';
 
-sub first_update ($$) {
+sub first_update {
+    @_ == 2 or fp_croak_nargs 2;
     my ($p, $fn) = @_;
     my ($v, $r)  = $p->first_and_rest;
     cons(&$fn($v), $r)
@@ -653,7 +663,8 @@ TEST {
 }
 bless [6, 4], 'FP::List::Pair';
 
-sub cdr ($) {
+sub cdr {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // die_not_a_pair($v);
     if ($r eq "FP::List::Pair") {
@@ -672,20 +683,46 @@ TEST { car cons(2, 3) } 2;
 TEST_EXCEPTION { car "FP::List::Pair" } "not a pair: 'FP::List::Pair'";
 TEST_EXCEPTION { cdr "FP::List::Pair" } "not a pair: 'FP::List::Pair'";
 
-sub rest ($);
-*rest = *cdr;
+sub rest;
+*rest = \&cdr;
 
-sub cddr ($)   { cdr cdr $_[0] }
-sub cdddr ($)  { cdr cdr cdr $_[0] }
-sub cddddr ($) { cdr cdr cdr cdr $_[0] }
+sub cddr {
+    @_ == 1 or fp_croak_nargs 1;
+    cdr cdr $_[0]
+}
 
-sub cadr ($)    { car cdr $_[0] }
-sub caddr ($)   { car cdr cdr $_[0] }
-sub cadddr ($)  { car cdr cdr cdr $_[0] }
-sub caddddr ($) { car cdr cdr cdr cdr $_[0] }
+sub cdddr {
+    @_ == 1 or fp_croak_nargs 1;
+    cdr cdr cdr $_[0]
+}
+
+sub cddddr {
+    @_ == 1 or fp_croak_nargs 1;
+    cdr cdr cdr cdr $_[0]
+}
+
+sub cadr {
+    @_ == 1 or fp_croak_nargs 1;
+    car cdr $_[0]
+}
+
+sub caddr {
+    @_ == 1 or fp_croak_nargs 1;
+    car cdr cdr $_[0]
+}
+
+sub cadddr {
+    @_ == 1 or fp_croak_nargs 1;
+    car cdr cdr cdr $_[0]
+}
+
+sub caddddr {
+    @_ == 1 or fp_croak_nargs 1;
+    car cdr cdr cdr cdr $_[0]
+}
 
 sub c_r {
-    @_ == 2 or die "wrong number of arguments";
+    @_ == 2 or fp_croak_nargs 2;
     my ($s, $chain) = @_;
     my $c;
     while (length($c = chop $chain)) {
@@ -697,12 +734,13 @@ sub c_r {
     $s
 }
 
-*FP::List::List::c_r = *c_r;
+*FP::List::List::c_r = \&c_r;
 
 TEST { list(1, list(4, 7, 9), 5)->c_r("addad") }
 9;
 
-sub car_and_cdr ($) {
+sub car_and_cdr {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // die_not_a_pair($v);
     if ($r eq "FP::List::Pair") {
@@ -715,10 +753,11 @@ sub car_and_cdr ($) {
     }
 }
 
-sub first_and_rest($);
-*first_and_rest = *car_and_cdr;
+sub first_and_rest;
+*first_and_rest = \&car_and_cdr;
 
-sub perhaps_first_and_rest ($) {
+sub perhaps_first_and_rest {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // die_not_a_pair($v);
     if ($r eq "FP::List::Pair") {
@@ -740,7 +779,8 @@ TEST { [perhaps_first_and_rest null] } [];
 TEST_EXCEPTION { [perhaps_first_and_rest "FP::List::Null"] }
 "not a pair: 'FP::List::Null'";    # and XX actually not a null either.
 
-sub list_perhaps_one ($) {
+sub list_perhaps_one {
+    @_ == 1 or fp_croak_nargs 1;
     my ($s) = @_;
     FORCE $s;                      # make work for stre
     if (is_pair($s)) {
@@ -752,13 +792,14 @@ sub list_perhaps_one ($) {
     }
 }
 
-*FP::List::List::perhaps_one = *list_perhaps_one;
+*FP::List::List::perhaps_one = \&list_perhaps_one;
 
 TEST { [list(8)->perhaps_one] } [8];
 TEST { [list(8, 9)->perhaps_one] } [];
 TEST { [list()->perhaps_one] } [];
 
-sub list_xone ($) {
+sub list_xone {
+    @_ == 1 or fp_croak_nargs 1;
     my ($s) = @_;
     FORCE $s;    # make work for streams
     if (is_pair($s)) {
@@ -773,7 +814,7 @@ sub list_xone ($) {
     }
 }
 
-*FP::List::List::xone = *list_xone;
+*FP::List::List::xone = \&list_xone;
 
 TEST { [list(8)->xone] } [8];
 TEST_EXCEPTION { [list(8, 9)->xone] } "expected 1 value, got more";
@@ -783,10 +824,11 @@ sub make_ref {
     my ($is_stream) = @_;
     my $liststream = $is_stream ? "stream" : "list";
 
-    sub ($ $) {
+    sub {
+        @_ == 2 or fp_croak_nargs 2;
         my ($s, $i) = @_;
         weaken $_[0] if $is_stream;
-        is_natural0 $i or die "invalid index: " . show($i);
+        is_natural0 $i or fp_croak "invalid index: " . show($i);
         my $orig_i = $i;
     LP: {
             $s = force $s;
@@ -812,9 +854,9 @@ sub make_ref {
     }
 }
 
-sub list_ref ($$);
+sub list_ref;
 *list_ref            = make_ref(0);
-*FP::List::List::ref = *list_ref;
+*FP::List::List::ref = \&list_ref;
 
 sub list {
     my $res = null;
@@ -909,6 +951,7 @@ TEST_EXCEPTION {
 'improper list';    # nice message at least, thanks to undef != null
 
 sub delayed (&) {
+    @_ == 1 or fp_croak_nargs 1;
     my ($thunk) = @_;
     sub {
         # evaluate thunk, expecting a function and pass our arguments
@@ -918,11 +961,10 @@ sub delayed (&) {
     }
 }
 
-sub list_of ($);
-
-sub list_of ($) {
+sub list_of {
+    @_ == 1 or fp_croak_nargs 1;
     my ($p) = @_;
-    either \&is_null, is_pair_of($p, delayed { list_of $p })
+    either \&is_null, is_pair_of($p, delayed { list_of($p) })
 }
 
 TEST { list_of(\&is_natural)->(list 1,  2, 3) } 1;
@@ -934,7 +976,8 @@ sub make_length {
     my ($is_stream) = @_;
     my $liststream = $is_stream ? "stream" : "list";
 
-    sub ($) {
+    sub {
+        @_ == 1 or fp_croak_nargs 1;
         my ($l) = @_;
         weaken $_[0] if $is_stream;
         my $len = 0;
@@ -956,10 +999,10 @@ sub make_length {
     }
 }
 
-sub list_length ($);
+sub list_length;
 *list_length = make_length(0);
 
-*FP::List::Pair::length = *list_length;
+*FP::List::Pair::length = \&list_length;
 
 # method on Pair not List, since we defined a length method for Null
 # explicitely
@@ -968,7 +1011,8 @@ TEST { list(4, 5, 6)->caddr } 6;
 TEST { list()->length } 0;
 TEST { list(4, 5)->length } 2;
 
-sub list_to_string ($) {
+sub list_to_string {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     my $len = list_length $l;
 
@@ -985,7 +1029,7 @@ sub list_to_string ($) {
     $res
 }
 
-*FP::List::List::string = *list_to_string;
+*FP::List::List::string = \&list_to_string;
 
 TEST { null->string } "";
 TEST { cons("a", null)->string } "a";
@@ -993,7 +1037,8 @@ TEST { list("Ha", "ll", "o")->string } "Hallo";
 TEST { list("", "", "o")->string } 'o';
 TEST { list("a", "", "o")->string } 'ao';
 
-sub list_to_array ($) {
+sub list_to_array {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     my $res = [];
     my $i   = 0;
@@ -1005,37 +1050,38 @@ sub list_to_array ($) {
     $res
 }
 
-*FP::List::List::array = *list_to_array;
+*FP::List::List::array = \&list_to_array;
 
 sub list_to_purearray {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     my $a = list_to_array $l;
     require FP::PureArray;
     FP::PureArray::array_to_purearray($a)
 }
 
-*FP::List::List::purearray = *list_to_purearray;
+*FP::List::List::purearray = \&list_to_purearray;
 
 TEST {
     list(1, 3, 4)->purearray->map (sub { $_[0]**2 })
 }
 bless [1, 9, 16], "FP::_::PureArray";
 
-sub list_sort ($;$) {
-    @_ == 1 or @_ == 2 or die "wrong number of arguments";
+sub list_sort {
+    @_ == 1 or @_ == 2 or fp_croak_nargs "1 or 2";
     my ($l, $maybe_cmp) = @_;
     list_to_purearray($l)->sort ($maybe_cmp)
 }
 
-*FP::List::List::sort = *list_sort;
+*FP::List::List::sort = \&list_sort;
 
-sub list_sortCompare ($) {
-    @_ == 1 or die "wrong number of arguments";
+sub list_sortCompare {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     list_to_purearray($l)->sortCompare
 }
 
-*FP::List::List::sortCompare = *list_sortCompare;
+*FP::List::List::sortCompare = \&list_sortCompare;
 
 TEST {
     require FP::Ops;
@@ -1056,7 +1102,8 @@ TEST { list(5, 3, 8, 4)->sort (\&FP::Ops::number_cmp)->first }
 TEST { list(5, 3, 8, 4)->sort (\&FP::Ops::number_cmp)->stream->car }
 3;
 
-sub rlist_to_array ($) {
+sub rlist_to_array {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     my $res = [];
     my $len = list_length $l;
@@ -1069,23 +1116,25 @@ sub rlist_to_array ($) {
     $res
 }
 
-*FP::List::List::reverse_array = *rlist_to_array;
+*FP::List::List::reverse_array = \&rlist_to_array;
 
-sub list_to_values ($) {
+sub list_to_values {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     @{ list_to_array($l) }
 }
 
-*FP::List::List::values = *list_to_values;
+*FP::List::List::values = \&list_to_values;
 
 # XX naming inconsistency versus docs/design.md ? Same with
 # rlist_to_array.
-sub rlist_to_values ($) {
+sub rlist_to_values {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     @{ rlist_to_array($l) }
 }
 
-*FP::List::List::reverse_values = *rlist_to_values;
+*FP::List::List::reverse_values = \&rlist_to_values;
 
 TEST { [list(3, 4, 5)->reverse_values] }
 [5, 4, 3];
@@ -1094,7 +1143,8 @@ sub make_for_each {
     my ($is_stream) = @_;
     my $liststream = $is_stream ? "stream" : "list";
 
-    sub ($ $ ) {
+    sub {
+        @_ == 2 or fp_croak_nargs 2;
         my ($proc, $s) = @_;
         weaken $_[1] if $is_stream;
     LP: {
@@ -1116,22 +1166,21 @@ sub make_for_each {
     }
 }
 
-sub list_for_each ($ $ );
+sub list_for_each;
 *list_for_each = make_for_each(1);
 
 *FP::List::List::for_each = flip \&list_for_each;
 
 TEST_STDOUT {
-    list(1, 3)->for_each(*xprintln)
+    list(1, 3)->for_each(\&xprintln)
 }
 "1\n3\n";
 
 # tons of slightly adapted COPIES from FP::Stream. XX finally find a
 # solution for this
 
-sub list_drop ($ $);
-
-sub list_drop ($ $) {
+sub list_drop {
+    @_ == 2 or fp_croak_nargs 2;
     my ($s, $n) = @_;
     while ($n > 0) {
         $s = force $s;
@@ -1142,25 +1191,23 @@ sub list_drop ($ $) {
     $s
 }
 
-*FP::List::List::drop = *list_drop;
+*FP::List::List::drop = \&list_drop;
 
-sub list_take ($ $);
-
-sub list_take ($ $) {
+sub list_take {
+    @_ == 2 or fp_croak_nargs 2;
     my ($s, $n) = @_;
     if ($n > 0) {
         $s = force $s;
-        is_null($s) ? $s : cons(car $s, list_take(cdr $s, $n - 1));
+        is_null($s) ? $s : cons(car($s), list_take(cdr($s), $n - 1));
     } else {
         null
     }
 }
 
-*FP::List::List::take = *list_take;
+*FP::List::List::take = \&list_take;
 
-sub list_slice ($ $);
-
-sub list_slice ($ $) {
+sub list_slice {
+    @_ == 2 or fp_croak_nargs 2;
     my ($start, $end) = @_;
     $end = force $end;
     my $rec;
@@ -1182,13 +1229,14 @@ sub list_slice ($ $) {
     goto &{ Weakened $rec};
 }
 
-*FP::List::List::slice = *list_slice;
+*FP::List::List::slice = \&list_slice;
 
 # maybe call it `cut_at` instead?
 
 # /COPIES
 
-sub string_to_list ($;$) {
+sub string_to_list {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($str, $maybe_tail) = @_;
     my $tail = $maybe_tail // null;
     my $i    = length($str) - 1;
@@ -1207,8 +1255,8 @@ TEST { list_to_string string_to_list "Hello" }
 'Hello';
 
 # XX HACK, COPY from FP::Array to work around circular dependency
-sub array_fold_right ($$$) {
-    @_ == 3 or die "wrong number of arguments";
+sub array_fold_right {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $tail, $a) = @_;
     my $i = @$a - 1;
     while ($i >= 0) {
@@ -1218,7 +1266,8 @@ sub array_fold_right ($$$) {
     $tail
 }
 
-sub array_fold ($$$) {
+sub array_fold {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $ary) = @_;
     for (@$ary) {
         $start = &$fn($_, $start);
@@ -1228,7 +1277,8 @@ sub array_fold ($$$) {
 
 # /HACK
 
-sub array_to_list ($;$) {
+sub array_to_list {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($a, $maybe_tail) = @_;
     array_fold_right(\&cons, $maybe_tail // null, $a)
 }
@@ -1237,7 +1287,8 @@ TEST { list_to_string array_to_list [1, 2, 3] }
 '123';
 
 # XX naming correct?
-sub array_to_list_reverse ($;$) {
+sub array_to_list_reverse {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($a, $maybe_tail) = @_;
     array_fold(\&cons, $maybe_tail // null, $a)
 }
@@ -1245,28 +1296,30 @@ sub array_to_list_reverse ($;$) {
 TEST { list_to_string array_to_list_reverse [1, 2, 3] }
 '321';
 
-sub list_reverse_with_tail ($$) {
+sub list_reverse_with_tail {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $tail) = @_;
     while (!is_null $l) {
-        $tail = cons car $l, $tail;
+        $tail = cons car($l), $tail;
         $l    = cdr $l;
     }
     $tail
 }
 
-sub list_reverse ($) {
+sub list_reverse {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     list_reverse_with_tail($l, $l->null)
 }
 
-*FP::List::List::reverse_with_tail = *list_reverse_with_tail;
-*FP::List::List::reverse           = *list_reverse;
+*FP::List::List::reverse_with_tail = \&list_reverse_with_tail;
+*FP::List::List::reverse           = \&list_reverse;
 
 TEST { list_to_string list_reverse string_to_list "Hello" }
 'olleH';
 
-sub list_strings_join ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub list_strings_join {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $val) = @_;
 
     # now depend on FP::Array anyway. Lazily. XX hack~
@@ -1274,13 +1327,13 @@ sub list_strings_join ($$) {
     FP::Array::array_strings_join(list_to_array($l), $val);
 }
 
-*FP::List::List::strings_join = *list_strings_join;
+*FP::List::List::strings_join = \&list_strings_join;
 
 TEST { list(1, 2, 3)->strings_join("-") }
 "1-2-3";
 
-sub list_strings_join_reverse ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub list_strings_join_reverse {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $val) = @_;
 
     # now depend on FP::Array anyway. Lazily. XX hack~
@@ -1288,22 +1341,21 @@ sub list_strings_join_reverse ($$) {
     FP::Array::array_strings_join(rlist_to_array($l), $val);
 }
 
-*FP::List::List::strings_join_reverse = *list_strings_join_reverse;
+*FP::List::List::strings_join_reverse = \&list_strings_join_reverse;
 
 TEST { list(1, 2, 3)->strings_join_reverse("-") }
 "3-2-1";
 
 # write as a S-expr (trying to follow R5RS Scheme)
-sub _write_sexpr ($ $ $);
-
-sub _write_sexpr ($ $ $) {
+sub _write_sexpr {
+    @_ == 3 or fp_croak_nargs 3;
     my ($l, $fh, $already_in_a_list) = @_;
 _WRITE_SEXPR: {
         $l = force($l, 1);
         if (is_pair $l) {
             xprint $fh, $already_in_a_list ? ' ' : '(';
-            _write_sexpr car $l, $fh, 0;
-            my $d = force(cdr $l, 1);
+            _write_sexpr(car($l), $fh, 0);
+            my $d = force(cdr($l), 1);
             if (is_null $d) {
                 xprint $fh, ')';
             } elsif (is_pair $d) {
@@ -1314,7 +1366,7 @@ _WRITE_SEXPR: {
                 redo _WRITE_SEXPR;
             } else {
                 xprint $fh, " . ";
-                _write_sexpr $d, $fh, 0;
+                _write_sexpr($d, $fh, 0);
                 xprint $fh, ')';
             }
         } elsif (is_null $l) {
@@ -1334,9 +1386,9 @@ _WRITE_SEXPR: {
         }
     }
 }
-sub write_sexpr ($ ; );
 
-sub write_sexpr ($ ; ) {
+sub write_sexpr {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l, $fh) = @_;
     _write_sexpr($l, $fh || *STDOUT{IO}, 0)
 }
@@ -1354,16 +1406,14 @@ TEST_STDOUT { write_sexpr(cons 1, 2) }
 TEST_STDOUT { write_sexpr cons(1, cons(cons(2, null), null)) }
 '("1" ("2"))';
 
-*FP::List::List::write_sexpr = *write_sexpr;
+*FP::List::List::write_sexpr = \&write_sexpr;
 
-sub list_zip2 ($$);
-
-sub list_zip2 ($$) {
-    @_ == 2 or die "expecting 2 arguments";
+sub list_zip2 {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $m) = @_;
     (     is_null($l) ? $l
         : is_null($m) ? $m
-        :               cons([car $l, car $m], list_zip2(cdr $l, cdr $m)))
+        :               cons([car($l), car($m)], list_zip2(cdr($l), cdr($m))))
 }
 
 TEST { list_to_array list_zip2 list(qw(a b c)), list(2, 3) }
@@ -1372,20 +1422,18 @@ TEST { list_to_array list_zip2 list(qw(a b c)), list(2, 3) }
 TEST { list_to_array list_zip2 list(qw(a b)), list(2, 3, 4) }
 [[a => 2], [b => 3]];
 
-*FP::List::List::zip = *list_zip2;    # XX make n-ary
+*FP::List::List::zip = \&list_zip2;    # XX make n-ary
 
-sub list_to_alist ($);
-
-sub list_to_alist ($) {
-    @_ == 1 or die "expecting 2 arguments";
+sub list_to_alist {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     is_null($l) ? $l : do {
         my ($k, $l2) = $l->first_and_rest;
         my ($v, $l3) = $l2->first_and_rest;
-        cons(cons($k, $v), list_to_alist $l3)
+        cons(cons($k, $v), list_to_alist($l3))
     }
 }
-*FP::List::List::alist = *list_to_alist;
+*FP::List::List::alist = \&list_to_alist;
 
 TEST_STDOUT { list(a => 10, b => 20)->alist->write_sexpr }
 '(("a" . "10") ("b" . "20"))';
@@ -1393,7 +1441,8 @@ TEST_STDOUT { list(a => 10, b => 20)->alist->write_sexpr }
 sub make_filter {
     my ($is_stream) = @_;
     my $filter;
-    $filter = sub ($$) {
+    $filter = sub {
+        @_ == 2 or fp_croak_nargs 2;
         my ($fn, $l) = @_;
         weaken $_[1] if $is_stream;
         lazy_if {
@@ -1410,7 +1459,7 @@ sub make_filter {
     Weakened($filter)
 }
 
-sub list_filter ($ $);
+sub list_filter;
 *list_filter = make_filter(0);
 
 *FP::List::List::filter = flip \&list_filter;
@@ -1419,7 +1468,8 @@ sub list_filter ($ $);
 sub make_filter_with_tail {
     my ($is_stream) = @_;
     my $filter_with_tail;
-    $filter_with_tail = sub ($$$) {
+    $filter_with_tail = sub {
+        @_ == 3 or fp_croak_nargs 3;
         my ($fn, $l, $tail) = @_;
         weaken $_[1] if $is_stream;
         lazy_if {
@@ -1435,15 +1485,14 @@ sub make_filter_with_tail {
     Weakened($filter_with_tail)
 }
 
-sub list_filter_with_tail ($$$);
+sub list_filter_with_tail;
 *list_filter_with_tail            = make_filter_with_tail(0);
-*FP::List::List::filter_with_tail = flip2of3 * list_filter_with_tail;
+*FP::List::List::filter_with_tail = flip2of3 \&list_filter_with_tail;
 
-sub list_map ($ $);
-
-sub list_map ($ $) {
+sub list_map {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
-    is_null($l) ? $l : cons(&$fn(car $l), list_map($fn, cdr $l))
+    is_null($l) ? $l : cons(scalar &$fn(car $l), list_map($fn, cdr $l))
 }
 
 TEST {
@@ -1472,7 +1521,7 @@ TEST {
 [[1, 'a'], [2, 'b']];
 
 sub FP::List::List::map {
-    @_ >= 2 or die "not enough arguments";
+    @_ >= 2 or fp_croak_nargs ">= 2";
     my $l  = shift;
     my $fn = shift;
     @_ ? list_mapn($fn, $l, @_) : list_map($fn, $l)
@@ -1489,12 +1538,12 @@ sub list_map_with_index_ {
 }
 
 sub list_map_with_index {
-    @_ >= 2 or die "not enough arguments";
+    @_ >= 2 or fp_croak_nargs ">= 2";
     list_map_with_index_(0, @_)
 }
 
 sub FP::List::List::map_with_index {
-    @_ >= 2 or die "not enough arguments";
+    @_ >= 2 or fp_croak_nargs ">= 2";
     my $l  = shift;
     my $fn = shift;
     list_map_with_index($fn, $l, @_)
@@ -1506,7 +1555,7 @@ TEST {
 [[0, 1], [1, 2], [2, 20]];
 
 sub list_map_with_islast {
-    @_ > 1 or die "wrong number of arguments";
+    @_ >= 2 or fp_croak_nargs ">= 2";
     my $fn      = shift;
     my @rest    = map { is_null($_) ? return null : rest $_ } @_;
     my $is_last = '';
@@ -1534,7 +1583,8 @@ TEST {
 
 # left fold, sometimes called `foldl` or `reduce`
 # (XX adapted copy from Stream.pm)
-sub list_fold ($$$) {
+sub list_fold {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $l) = @_;
     my $v;
 LP: {
@@ -1557,14 +1607,13 @@ TEST {
 }
 9;
 
-sub list_fold_right ($ $ $);
-
-sub list_fold_right ($ $ $) {
+sub list_fold_right {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $l) = @_;
     if (is_pair $l) {
         no warnings 'recursion';
         my $rest = list_fold_right($fn, $start, cdr $l);
-        &$fn(car $l, $rest)
+        &$fn(car($l), $rest)
     } elsif (is_null $l) {
         $start
     } else {
@@ -1582,7 +1631,7 @@ TEST {
 
 sub FP::List::List::fold_right {
     my $l = shift;
-    @_ == 2 or die "expecting 2 arguments";
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $start) = @_;
     list_fold_right($fn, $start, $l)
 }
@@ -1594,10 +1643,8 @@ TEST {
 
 # same as fold_right but passes the whole list remainder instead of
 # only the car to the function
-sub list_pair_fold_right ($$$);
-
-sub list_pair_fold_right ($$$) {
-    @_ == 3 or die "wrong number of arguments";
+sub list_pair_fold_right {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $l) = @_;
     if (is_pair $l) {
         no warnings 'recursion';
@@ -1610,9 +1657,9 @@ sub list_pair_fold_right ($$$) {
     }
 }
 
-*FP::List::List::pair_fold_right = rot3left * list_pair_fold_right;
+*FP::List::List::pair_fold_right = rot3left \&list_pair_fold_right;
 
-TEST_STDOUT { list(5, 6, 9)->pair_fold_right(*cons, null)->write_sexpr }
+TEST_STDOUT { list(5, 6, 9)->pair_fold_right(\&cons, null)->write_sexpr }
 '(("5" "6" "9") ("6" "9") ("9"))';
 
 # no list_ prefix? It doesn't consume lists. Although, still FP::List
@@ -1632,25 +1679,21 @@ TEST_STDOUT { list(5, 6, 9)->pair_fold_right(*cons, null)->write_sexpr }
 # For more documentation, see
 # http://srfi.schemers.org/srfi-1/srfi-1.html#FoldUnfoldMap
 
-sub unfold ($$$$;$);
-
-sub unfold ($$$$;$) {
-    @_ == 4 or @_ == 5 or die "wrong number of arguments";
+sub unfold {
+    @_ == 4 or @_ == 5 or fp_croak_nargs "4 or 5";
     my ($p, $f, $g, $seed, $maybe_tail_gen) = @_;
     &$p($seed)
         ? (defined $maybe_tail_gen ? &$maybe_tail_gen($seed) : null)
         : cons(&$f($seed), unfold($p, $f, $g, &$g($seed), $maybe_tail_gen));
 }
 
-TEST { unfold(*is_zero, *inc, *dec, 5)->array } [6, 5, 4, 3, 2];
-TEST { unfold(*is_zero, *inc, *dec, 5, *list)->array } [6, 5, 4, 3, 2, 0];
+TEST { unfold(\&is_zero, \&inc, \&dec, 5)->array } [6, 5, 4, 3, 2];
+TEST { unfold(\&is_zero, \&inc, \&dec, 5, \&list)->array } [6, 5, 4, 3, 2, 0];
 
 # unfold-right p f g seed [tail] -> list
 
-sub unfold_right ($$$$;$);
-
-sub unfold_right ($$$$;$) {
-    @_ == 4 or @_ == 5 or die "wrong number of arguments";
+sub unfold_right {
+    @_ == 4 or @_ == 5 or fp_croak_nargs "4 or 5";
     my ($p, $f, $g, $seed, $maybe_tail) = @_;
     my $tail = @_ == 5 ? $maybe_tail : null;
 LP: {
@@ -1663,8 +1706,8 @@ LP: {
     }
 }
 
-TEST { unfold_right(*is_zero, *inc, *dec, 5)->array } [2, 3, 4, 5, 6];
-TEST { unfold_right(*is_zero, *inc, *dec, 5, list 99)->array }
+TEST { unfold_right(\&is_zero, \&inc, \&dec, 5)->array } [2, 3, 4, 5, 6];
+TEST { unfold_right(\&is_zero, \&inc, \&dec, 5, list 99)->array }
 [2, 3, 4, 5, 6, 99];
 
 sub list_append {
@@ -1688,12 +1731,13 @@ TEST {
 }
 ['a', 'b', 1, 2];
 
-*FP::List::List::append = *list_append;
+*FP::List::List::append = \&list_append;
 
 TEST { array_to_list(["a", "b"])->append(array_to_list([1, 2]))->array }
 ['a', 'b', 1, 2];
 
-sub list_to_perlstring ($) {
+sub list_to_perlstring {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     list_to_string cons(
         "'",
@@ -1716,11 +1760,10 @@ TEST { list_to_perlstring string_to_list "Hello" }
 TEST { list_to_perlstring string_to_list "Hello's" }
 q{'Hello\'s'};
 
-*FP::List::List::perlstring = *list_to_perlstring;
+*FP::List::List::perlstring = \&list_to_perlstring;
 
-sub list_butlast ($);
-
-sub list_butlast ($) {
+sub list_butlast {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     if (is_null($l)) {
         die "butlast: got empty list"
@@ -1728,18 +1771,19 @@ sub list_butlast ($) {
             # XX could make use of OO for the distinction instead
     } else {
         my ($a, $r) = $l->first_and_rest;
-        is_null($r) ? $r : cons($a, list_butlast $r)
+        is_null($r) ? $r : cons($a, list_butlast($r))
     }
 }
 
-*FP::List::List::butlast = *list_butlast;
+*FP::List::List::butlast = \&list_butlast;
 
 TEST { list(3, 4, 5)->butlast->array }
 [3, 4];
 TEST_EXCEPTION { list()->butlast->array }
 'can\'t take the butlast of the empty list';
 
-sub list_drop_while ($ $) {
+sub list_drop_while {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l) = @_;
     while (!is_null $l and &$pred(car $l)) {
         $l = cdr $l;
@@ -1765,23 +1809,25 @@ TEST {
 }
 "o World";
 
-sub list_rtake_while_and_rest ($ $) {
+sub list_rtake_while_and_rest {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l) = @_;
     my $res = $l->null;
     my $c;
-    while (!is_null $l and &$pred($c = car $l)) {
+    while (!is_null $l and &$pred($c = $l->car)) {
         $res = cons $c, $res;
-        $l   = cdr $l;
+        $l   = $l->cdr;
     }
     ($res, $l)
 }
 
 *FP::List::List::rtake_while_and_rest = flip \&list_rtake_while_and_rest;
 
-sub list_rtake_while ($ $) {
+sub list_rtake_while {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l)    = @_;
     my ($res,  $rest) = list_rtake_while_and_rest($pred, $l);
-    wantarray ? ($res, $rest) : $res
+    $res
 }
 
 *FP::List::List::rtake_while = flip \&list_rtake_while;
@@ -1792,18 +1838,20 @@ TEST {
 }
 'Hello';
 
-sub list_take_while_and_rest ($ $) {
+sub list_take_while_and_rest {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l)    = @_;
-    my ($rres, $rest) = list_rtake_while($pred, $l);
-    (list_reverse $rres, $rest)
+    my ($rres, $rest) = list_rtake_while_and_rest($pred, $l);
+    (list_reverse($rres), $rest)
 }
 
 *FP::List::List::take_while_and_rest = flip \&list_take_while_and_rest;
 
-sub list_take_while ($ $) {
+sub list_take_while {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l)    = @_;
     my ($res,  $rest) = list_take_while_and_rest($pred, $l);
-    wantarray ? ($res, $rest) : $res
+    $res
 }
 
 *FP::List::List::take_while = flip \&list_take_while;
@@ -1827,7 +1875,8 @@ TEST {
 }
 "";
 
-sub list_last ($) {
+sub list_last {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
 LIST_LAST: {
         my ($a, $r) = $v->first_and_rest;
@@ -1851,7 +1900,8 @@ TEST_EXCEPTION { list(qw())->last }
 
 # XX: add list_last_pair (see SRFI 1)
 
-sub list_every ($$) {
+sub list_every {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l) = @_;
 LP: {
         if (is_pair $l) {
@@ -1875,8 +1925,8 @@ LP: {
 
 # XXX do we want this alias? Or do we just want to rename every to
 # all?
-sub list_all ($$);
-*list_all = *list_every;
+sub list_all;
+*list_all = \&list_every;
 
 *FP::List::List::all = flip \&list_every;
 
@@ -1918,7 +1968,8 @@ TEST {
 }
 [2, 0];
 
-sub list_any ($ $) {
+sub list_any {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l) = @_;
 LP: {
         if (is_pair $l) {
@@ -1957,8 +2008,8 @@ TEST {
 # in that they do not return false or undef on failure, but (), and
 # carry `perhaps` in their name of this reason.
 
-sub list_perhaps_find_tail ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub list_perhaps_find_tail {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
 LP: {
         if (is_null $l) { () }
@@ -1977,14 +2028,14 @@ LP: {
 *FP::List::List::perhaps_find_tail = flip \&list_perhaps_find_tail;
 
 TEST {
-    list(3, 1, 37, -8, -5, 0, 0)->perhaps_find_tail(*is_even)->array
+    list(3, 1, 37, -8, -5, 0, 0)->perhaps_find_tail(\&is_even)->array
 }
 [-8, -5, 0, 0];
-TEST { [list(3, 1, 37, -5)->perhaps_find_tail(*is_even)] }
+TEST { [list(3, 1, 37, -5)->perhaps_find_tail(\&is_even)] }
 [];
 
-sub list_perhaps_find ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub list_perhaps_find {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     if (my ($l) = list_perhaps_find_tail($fn, $l)) {
         $l->car
@@ -1995,34 +2046,33 @@ sub list_perhaps_find ($$) {
 
 *FP::List::List::perhaps_find = flip \&list_perhaps_find;
 
-TEST { list(3, 1, 4, 1, 5, 9)->perhaps_find(*is_even) }
+TEST { list(3, 1, 4, 1, 5, 9)->perhaps_find(\&is_even) }
 4;
 
 # And then still also add the SRFI-1 counterparts, without `maybe` in
 # the names as they should have according to our guidelines, XX hmm.
 
-#sub list_find_tail ($$);
-#  sigh, can't retain the prototypes unless writing perhaps_to_maybe
-#  for every number of arguments.
+sub list_find_tail;
 *list_find_tail            = perhaps_to_maybe(\&list_perhaps_find_tail);
 *FP::List::List::find_tail = flip \&list_find_tail;
 
-#sub list_find ($$);
+sub list_find;
 *list_find            = perhaps_to_maybe(\&list_perhaps_find);
 *FP::List::List::find = flip \&list_find;
 
-TEST { list(3, 1, 4, 1, 5, 9)->find(*is_even) }
+TEST { list(3, 1, 4, 1, 5, 9)->find(\&is_even) }
 4;
-TEST { list(3, 1, 37, -8, -5, 0, 0)->find_tail(*is_even)->array }
+TEST { list(3, 1, 37, -8, -5, 0, 0)->find_tail(\&is_even)->array }
 [-8, -5, 0, 0];
-TEST { [list(3, 1, 37, -5)->find_tail(*is_even)] }
+TEST { [list(3, 1, 37, -5)->find_tail(\&is_even)] }
 [undef];
 
 # Grouping
 
 sub make_group {
     my ($is_stream) = @_;
-    my $group = sub ($$;$) {
+    my $group = sub {
+        @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
         my ($equal, $s, $maybe_tail) = @_;
         weaken $_[1] if $is_stream;
         lazy_if {
@@ -2065,17 +2115,17 @@ sub make_group {
     }
 }
 
-sub list_group ($$;$);
+sub list_group;
 *list_group = make_group(0);
 
 sub FP::List::List::group {
-    @_ >= 2 and @_ <= 3 or die "wrong number of arguments";
+    @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
     my ($self, $equal, $maybe_tail) = @_;
     list_group($equal, $self, $maybe_tail)
 }
 
 TEST {
-    list(3, 4, 4, 5, 6, 8, 5, 5)->group(*FP::Ops::number_eq)
+    list(3, 4, 4, 5, 6, 8, 5, 5)->group(\&FP::Ops::number_eq)
 }
 list(list(3), list(4, 4), list(5), list(6), list(8), list(5, 5));
 
@@ -2085,9 +2135,8 @@ list(list(3), list(4, 4), list(5), list(6), list(8), list(5, 5));
 # lazy or lazyLight. In that case it will force promises, but only
 # lazily (i.e. provide a promise that will do the forcing and consing).
 
-sub mixed_flatten ($;$$);
-
-sub mixed_flatten ($;$$) {
+sub mixed_flatten {
+    @_ >= 1 and @_ <= 3 or fp_croak_nargs "1-3";
     my ($v, $maybe_tail, $maybe_delay) = @_;
     my $tail = $maybe_tail // null;
 LP: {
@@ -2104,13 +2153,13 @@ LP: {
                 $tail
             } elsif (is_pair $v) {
                 no warnings 'recursion';
-                $tail = mixed_flatten(cdr $v, $tail, $maybe_delay);
+                $tail = mixed_flatten(cdr($v), $tail, $maybe_delay);
                 $v    = car $v;
                 redo LP;
             } elsif (ref $v eq "ARRAY") {
                 @_ = (
                     sub {
-                        @_ == 2 or die "wrong number of arguments";
+                        @_ == 2 or fp_croak_nargs 2;
                         my ($v, $tail) = @_;
                         no warnings 'recursion';
 
@@ -2214,15 +2263,15 @@ TEST_STDOUT {
 
 use FP::Char 'is_char';
 
-sub is_charlist ($) {
+sub is_charlist {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     list_every \&is_char, $l
 }
 
-*FP::List::List::is_charlist = *is_charlist;
-
-use Carp;
+*FP::List::List::is_charlist = \&is_charlist;
 
+# XX update to use fp_croak ?
 sub ldie {
 
     # perl string arguments are messages, char lists are turned to
