@@ -65,8 +65,10 @@ our @EXPORT_OK   = qw();
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
 use Scalar::Util 'blessed';
+use FP::Carp;
 
-sub is_mutablearray ($) {
+sub is_mutablearray {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     ($r eq "FP::_::MutableArray" or $v->isa("FP::_::MutableArray"))
@@ -76,7 +78,8 @@ sub mutablearray {
     FP::_::MutableArray->new_from_array([@_])
 }
 
-sub array_to_purearray ($) {
+sub array_to_purearray {
+    @_ == 1 or fp_croak_nargs 1;
     FP::_::MutableArray->new_from_array($_[0])
 }
 
@@ -84,22 +87,23 @@ package FP::_::MutableArray {
     use base qw(FP::Array::Mixin);
     use FP::Interfaces;
     use Carp;
+    use FP::Carp;
     use Chj::NamespaceCleanAbove;
 
     sub new_from_array {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($class, $a) = @_;
         bless $a, $class
     }
 
     sub mutablearray {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         $s
     }
 
     sub purearray {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         FP::_::PureArray->new_from_array($s)
     }
@@ -117,7 +121,7 @@ package FP::_::MutableArray {
     }
 
     sub pure {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
 
         # same as `array_to_purearray`
         require FP::PureArray;    # cost?
@@ -127,7 +131,7 @@ package FP::_::MutableArray {
     our $unsafe_mutable_warned = 0;
 
     sub unsafe_mutable {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $a = shift;
         carp "is already mutable" unless $unsafe_mutable_warned++;
         $a
