@@ -73,6 +73,7 @@ our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 use FP::Docstring;
 use FP::Show;
 use Scalar::Util qw(blessed);
+use FP::Carp;
 
 our $immutable = 1;
 
@@ -97,7 +98,8 @@ sub purehash {
     $res
 }
 
-sub is_purehash ($) {
+sub is_purehash {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     $v->isa("FP::_::PureHash")
@@ -132,25 +134,26 @@ package FP::Hash::Mixin {
 package FP::_::PureHash {
     use base "FP::Hash::Mixin";
     use FP::Interfaces;
+    use FP::Carp;
     use Chj::NamespaceCleanAbove;
 
     sub constructor_name {"purehash"}
 
     # XX  why not get, again? set and get? If ref, then what for set?
     sub ref {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($s, $key) = @_;
         $$s{$key}
     }
 
     sub perhaps_ref {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($s, $key) = @_;
         exists $$s{$key} ? $$s{$key} : ()
     }
 
     sub set {
-        @_ == 3 or die "wrong number of arguments";
+        @_ == 3 or fp_croak_nargs 3;
         my ($s, $key, $val) = @_;
 
         # XX the inefficient approach...  to be replaced with new impl.
