@@ -179,9 +179,11 @@ use FP::Weak;
 use FP::Predicates 'is_natural0';
 use FP::Show;
 use FP::fix;
+use Carp;
+use FP::Carp;
 
 sub stream_iota {
-    @_ <= 2 or die "wrong number of arguments";
+    @_ <= 2 or fp_croak_nargs 2;
     my ($maybe_start, $maybe_n) = @_;
     my $start = $maybe_start // 0;
     if (defined $maybe_n) {
@@ -216,7 +218,7 @@ sub stream_iota {
 
 # Like perl's `..`
 sub stream_range {
-    @_ <= 2 or die "wrong number of arguments";
+    @_ <= 2 or fp_croak_nargs 2;
     my ($maybe_start, $maybe_end) = @_;
     my $start = $maybe_start // 0;
     stream_iota($start, defined $maybe_end ? $maybe_end + 1 - $start : undef);
@@ -230,7 +232,7 @@ TEST { stream_range(2, 1)->array }
 [];
 
 sub stream_step_range {
-    (@_ >= 1 and @_ <= 3) or die "wrong number of arguments";
+    (@_ >= 1 and @_ <= 3) or fp_croak_nargs "1-3";
     my ($step, $maybe_start, $maybe_end) = @_;
     my $start   = $maybe_start // 0;
     my $inverse = $step < 0;
@@ -273,7 +275,7 @@ TEST { stream_step_range(2, 3, 7)->array }
 TEST { stream_step_range(-1, 3, 1)->array }
 [3, 2, 1];
 
-sub stream_length ($);
+sub stream_length;
 *stream_length = FP::List::make_length(1);
 
 *FP::List::List::stream_length = *stream_length;
@@ -307,7 +309,8 @@ TEST {
 # fold. If nothing more this allows `*cons` to be passed as $fn
 # directly to construct a list.
 
-sub stream_fold ($$$) {
+sub stream_fold {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $l) = @_;
     weaken $_[2];
     my $v;
@@ -322,7 +325,7 @@ LP: {
     $start
 }
 
-*FP::List::List::stream_fold = rot3left * stream_fold;
+*FP::List::List::stream_fold = rot3left \&stream_fold;
 
 TEST {
     stream_fold sub { $_[0] + $_[1] }, 5, stream_iota(10, 2)
@@ -339,7 +342,8 @@ TEST {
 TEST { stream_fold(\&cons, null, stream(1, 2))->array }
 [2, 1];
 
-sub stream_sum ($) {
+sub stream_sum {
+    @_ == 1 or fp_croak_nargs 1;
     my ($s) = @_;
     weaken $_[0];
     stream_fold(sub { $_[0] + $_[1] }, 0, $s)
@@ -357,14 +361,14 @@ TEST {
 #TEST{ stream_iota->rest->take(100000)->sum }
 #  5000050000;
 
-sub stream_append2 ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub stream_append2 {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l1, $l2) = @_;
     weaken $_[0];
     weaken $_[1];
     lazy {
         $l1 = force $l1;
-        is_null($l1) ? $l2 : cons(car $l1, stream_append(cdr $l1, $l2))
+        is_null($l1) ? $l2 : cons(car($l1), stream_append(cdr($l1), $l2))
     }
 }
 
@@ -442,9 +446,8 @@ TEST {
     ["take 6", "a", "b", "c"]
 ];
 
-sub stream_map ($ $);
-
-sub stream_map ($ $) {
+sub stream_map {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     weaken $_[1];
     lazy {
@@ -453,27 +456,25 @@ sub stream_map ($ $) {
     }
 }
 
-*FP::List::List::stream_map = flip * stream_map;
+*FP::List::List::stream_map = flip \&stream_map;
 
-sub stream_map_with_tail ($ $ $);
-
-sub stream_map_with_tail ($ $ $) {
+sub stream_map_with_tail {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $l, $tail) = @_;
     weaken $_[1];
     lazy {
         $l = force $l;
         is_null($l)
             ? $tail
-            : cons(&$fn(car $l), stream_map_with_tail($fn, cdr $l, $tail))
+            : cons(&$fn(car $l), stream_map_with_tail($fn, cdr($l), $tail))
     }
 }
 
-*FP::List::List::stream_map_with_tail = flip2of3 * stream_map_with_tail;
+*FP::List::List::stream_map_with_tail = flip2of3 \&stream_map_with_tail;
 
 # 2-ary (possibly slightly faster) version of stream_zip
-sub stream_zip2 ($$);
-
-sub stream_zip2 ($$) {
+sub stream_zip2 {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $m) = @_;
     do { weaken $_ if is_promise $_ }
         for @_;    #needed?
@@ -482,7 +483,7 @@ sub stream_zip2 ($$) {
         $m = force $m;
         (is_null $l or is_null $m)
             ? null
-            : cons([car $l, car $m], stream_zip2(cdr $l, cdr $m))
+            : cons([car($l), car($m)], stream_zip2(cdr($l), cdr($m)))
     }
 }
 
@@ -514,35 +515,34 @@ sub stream_zip_with {
     lazy {
         my $l1 = force $l1;
         my $l2 = force $l2;
-        (is_null $l1 or is_null $l2) ? null : cons &$f(car $l1, car $l2),
-            stream_zip_with($f, cdr $l1, cdr $l2)
+        (is_null $l1 or is_null $l2) ? null : cons &$f(car($l1), car($l2)),
+            stream_zip_with($f, cdr($l1), cdr($l2))
     }
 }
 
-*FP::List::List::stream_zip_with = flip2of3 * stream_zip_with;
+*FP::List::List::stream_zip_with = flip2of3 \&stream_zip_with;
 
-sub stream_filter ($ $);
+sub stream_filter;
 *stream_filter                 = FP::List::make_filter(1);
-*FP::List::List::stream_filter = flip * stream_filter;
+*FP::List::List::stream_filter = flip \&stream_filter;
 
-sub stream_filter_with_tail ($$$);
+sub stream_filter_with_tail;
 *stream_filter_with_tail                 = FP::List::make_filter_with_tail(1);
-*FP::List::List::stream_filter_with_tail = flip2of3 * stream_filter_with_tail;
+*FP::List::List::stream_filter_with_tail = flip2of3 \&stream_filter_with_tail;
 
 # http://hackage.haskell.org/package/base-4.7.0.2/docs/Prelude.html#v:foldr1
 
 # foldr1 is a variant of foldr that has no starting value argument,
 # and thus must be applied to non-empty lists.
 
-sub stream_foldr1 ($ $);
-
-sub stream_foldr1 ($ $) {
+sub stream_foldr1 {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     weaken $_[1];
     lazy {
         $l = force $l;
         if (is_pair $l) {
-            &$fn(car $l, stream_foldr1($fn, cdr $l))
+            &$fn(car($l), stream_foldr1($fn, cdr($l)))
         } elsif (is_null $l) {
             die "foldr1: reached end of list"
         } else {
@@ -551,17 +551,16 @@ sub stream_foldr1 ($ $) {
     }
 }
 
-*FP::List::List::stream_foldr1 = flip * stream_foldr1;
+*FP::List::List::stream_foldr1 = flip \&stream_foldr1;
 
-sub stream_fold_right ($ $ $);
-
-sub stream_fold_right ($ $ $) {
+sub stream_fold_right {
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $start, $l) = @_;
     weaken $_[2];
     lazy {
         $l = force $l;
         if (is_pair $l) {
-            &$fn(car $l, stream_fold_right($fn, $start, cdr $l))
+            &$fn(car($l), stream_fold_right($fn, $start, cdr $l))
         } elsif (is_null $l) {
             $start
         } else {
@@ -570,15 +569,15 @@ sub stream_fold_right ($ $ $) {
     }
 }
 
-*FP::List::List::stream_fold_right = rot3left * stream_fold_right;
+*FP::List::List::stream_fold_right = rot3left \&stream_fold_right;
 
 *FP::List::List::stream_preferred_fold = *FP::List::List::stream_fold_right;
 
 sub make_stream__fold_right {
     my ($length, $ref, $start, $d, $whileP) = @_;
 
-    sub ($$$) {
-        @_ == 3 or die "wrong number of arguments";
+    sub {
+        @_ == 3 or fp_croak_nargs 3;
         my ($fn, $tail, $a) = @_;
         my $len = &$length($a);
         my $rec;
@@ -604,24 +603,26 @@ our $array_ref     = sub { $_[0][$_[1]] };
 our $string_length = sub { length $_[0] };
 our $string_ref    = sub { substr $_[0], $_[1], 1 };
 
-sub stream__array_fold_right ($$$);
+sub stream__array_fold_right;
 *stream__array_fold_right
     = make_stream__fold_right($array_length, $array_ref, 0, 1, $lt);
 
 # XX export these array functions as methods to ARRAY library
 
-sub stream__string_fold_right ($$$);
+sub stream__string_fold_right;
 *stream__string_fold_right
     = make_stream__fold_right($string_length, $string_ref, 0, 1, $lt);
 
-sub stream__subarray_fold_right ($$$$$) {
+sub stream__subarray_fold_right {
+    @_ == 5 or fp_croak_nargs 5;
     my ($fn, $tail, $a, $start, $maybe_end) = @_;
     make_stream__fold_right($array_length, $array_ref, $start, 1,
         defined $maybe_end ? sub { $_[0] < $_[1] and $_[0] < $maybe_end } : $lt)
         ->($fn, $tail, $a);
 }
 
-sub stream__subarray_fold_right_reverse ($$$$$) {
+sub stream__subarray_fold_right_reverse {
+    @_ == 5 or fp_croak_nargs 5;
     my ($fn, $tail, $a, $start, $maybe_end) = @_;
     make_stream__fold_right($array_length, $array_ref, $start, -1,
         defined $maybe_end
@@ -629,7 +630,8 @@ sub stream__subarray_fold_right_reverse ($$$$$) {
         : sub { $_[0] >= 0 })->($fn, $tail, $a);
 }
 
-sub array_to_stream ($;$) {
+sub array_to_stream {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($a, $maybe_tail) = @_;
     stream__array_fold_right(*cons, $maybe_tail // null, $a)
 }
@@ -638,24 +640,28 @@ sub stream {
     array_to_stream [@_]
 }
 
-sub subarray_to_stream ($$;$$) {
+sub subarray_to_stream {
+    @_ >= 2 and @_ <= 4 or fp_croak_nargs "2-4";
     my ($a, $start, $maybe_end, $maybe_tail) = @_;
     stream__subarray_fold_right(*cons, $maybe_tail // null,
         $a, $start, $maybe_end)
 }
 
-sub subarray_to_stream_reverse ($$;$$) {
+sub subarray_to_stream_reverse {
+    @_ >= 2 and @_ <= 4 or fp_croak_nargs "2-4";
     my ($a, $start, $maybe_end, $maybe_tail) = @_;
     stream__subarray_fold_right_reverse(*cons, $maybe_tail // null,
         $a, $start, $maybe_end)
 }
 
-sub string_to_stream ($;$) {
+sub string_to_stream {
+    @_ >= 1 and @_ <= 2 or fp_croak_nargs "1-2";
     my ($str, $maybe_tail) = @_;
     stream__string_fold_right(*cons, $maybe_tail // null, $str)
 }
 
-sub stream_to_string ($) {
+sub stream_to_string {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     weaken $_[0];
     my $str = "";
@@ -670,8 +676,8 @@ sub stream_to_string ($) {
 
 TEST { stream("Ha", "ll", "o")->string } "Hallo";
 
-sub stream_strings_join ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub stream_strings_join {
+    @_ == 2 or fp_croak_nargs 2;
     my ($l, $val) = @_;
     weaken $_[0];
 
@@ -688,14 +694,13 @@ sub stream_strings_join ($$) {
 TEST { stream(1, 2, 3)->strings_join("-") }
 "1-2-3";
 
-sub stream_for_each ($ $ );
+sub stream_for_each;
 *stream_for_each = FP::List::make_for_each(1);
 
-*FP::List::List::stream_for_each = flip * stream_for_each;
+*FP::List::List::stream_for_each = flip \&stream_for_each;
 
-sub stream_drop ($ $);
-
-sub stream_drop ($ $) {
+sub stream_drop {
+    @_ == 2 or fp_croak_nargs 2;
     my ($s, $n) = @_;
     weaken $_[0];
     while ($n > 0) {
@@ -709,15 +714,14 @@ sub stream_drop ($ $) {
 
 *FP::List::List::stream_drop = *stream_drop;
 
-sub stream_take ($ $);
-
-sub stream_take ($ $) {
+sub stream_take {
+    @_ == 2 or fp_croak_nargs 2;
     my ($s, $n) = @_;
     weaken $_[0];
     lazy {
         if ($n > 0) {
             $s = force $s;
-            is_null($s) ? $s : cons(car $s, stream_take(cdr $s, $n - 1));
+            is_null($s) ? $s : cons(car($s), stream_take(cdr($s), $n - 1));
         } else {
             null
         }
@@ -726,9 +730,8 @@ sub stream_take ($ $) {
 
 *FP::List::List::stream_take = *stream_take;
 
-sub stream_take_while ($ $);
-
-sub stream_take_while ($ $) {
+sub stream_take_while {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $s) = @_;
     weaken $_[1];
     lazy {
@@ -746,11 +749,10 @@ sub stream_take_while ($ $) {
     }
 }
 
-*FP::List::List::stream_take_while = flip * stream_take_while;
+*FP::List::List::stream_take_while = flip \&stream_take_while;
 
-sub stream_slice ($ $);
-
-sub stream_slice ($ $) {
+sub stream_slice {
+    @_ == 2 or fp_croak_nargs 2;
     my ($start, $end) = @_;
     weaken $_[0];
     weaken $_[1];
@@ -781,7 +783,8 @@ sub stream_slice ($ $) {
 
 # maybe call it `cut_at` instead?
 
-sub stream_drop_while ($ $) {
+sub stream_drop_while {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $s) = @_;
     weaken $_[1];
     lazy {
@@ -797,13 +800,14 @@ sub stream_drop_while ($ $) {
     }
 }
 
-*FP::List::List::stream_drop_while = flip * stream_drop_while;
+*FP::List::List::stream_drop_while = flip \&stream_drop_while;
 
-sub stream_ref ($$);
+sub stream_ref;
 *stream_ref                 = FP::List::make_ref(1);
 *FP::List::List::stream_ref = *stream_ref;
 
 sub exn (&) {
+    @_ == 1 or fp_croak_nargs 1;
     my ($thunk) = @_;
     eval { &$thunk(); '' } // do { $@ =~ /(.*?) at/; $1 }
 }
@@ -839,15 +843,14 @@ TEST { list_ref cons(0, stream(1, 2, 3)), 2 } 2;
 TEST { stream_ref cons(0, stream(1, 2, 3)), 2 } 2;
 
 # force everything deeply
-sub F ($);
-
-sub F ($) {
+sub F {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
 
     #weaken $_[0]; since I usually use it interactively, and should
     # only be good for short sequences, better don't
     if (is_promise $v) {
-        F force $v;
+        F(force $v);
     } else {
         if (length(my $r = ref $v)) {
             if (is_pair $v) {
@@ -855,9 +858,9 @@ sub F ($) {
             } elsif (is_null $v) {
                 $v
             } elsif ($r eq "ARRAY") {
-                [map { F $_ } @$v]
+                [map { F($_) } @$v]
             } elsif (UNIVERSAL::isa($v, "ARRAY")) {
-                bless [map { F $_ } @$v], ref $v
+                bless [map { F($_) } @$v], ref $v
             } else {
                 $v
             }
@@ -867,7 +870,8 @@ sub F ($) {
     }
 }
 
-sub stream_to_array ($) {
+sub stream_to_array {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     weaken $_[0];
     my $res = [];
@@ -899,7 +903,8 @@ TEST {
 }
 bless [1, 9, 16], "FP::_::PureArray";
 
-sub stream_to_list ($) {
+sub stream_to_list {
+    @_ == 1 or fp_croak_nargs 1;
     my ($l) = @_;
     weaken $_[0];
     array_to_list stream_to_array $l
@@ -907,8 +912,8 @@ sub stream_to_list ($) {
 
 *FP::List::List::stream_list = *stream_to_list;
 
-sub stream_sort ($;$) {
-    @_ == 1 or @_ == 2 or die "wrong number of arguments";
+sub stream_sort {
+    @_ == 1 or @_ == 2 or fp_croak_nargs "1 or 2";
     my ($l, $maybe_cmp) = @_;
     stream_to_purearray($l)->sort ($maybe_cmp)
 }
@@ -934,17 +939,18 @@ TEST { stream(5, 3, 10, 8, 4)->sort->stream->car }
 
 # add a lazy merge sort instead/in addition?
 
-sub stream_group ($$;$);
+sub stream_group;
 *stream_group = FP::List::make_group(1);
 
 sub FP::List::List::stream_group {
-    @_ >= 2 and @_ <= 3 or die "wrong number of arguments";
+    @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
     my ($self, $equal, $maybe_tail) = @_;
     weaken $_[0];
     stream_group($equal, $self, $maybe_tail)
 }
 
-sub stream_mixed_flatten ($;$$) {
+sub stream_mixed_flatten {
+    @_ >= 1 and @_ <= 3 or fp_croak_nargs "1-3";
     my ($v, $maybe_tail, $maybe_delay) = @_;
 
     #XXX needed, no? weaken $_[0] if ref $_[0];
@@ -953,9 +959,8 @@ sub stream_mixed_flatten ($;$$) {
 
 *FP::List::List::stream_mixed_flatten = *stream_mixed_flatten;
 
-sub stream_any ($ $);
-
-sub stream_any ($ $) {
+sub stream_any {
+    @_ == 2 or fp_croak_nargs 2;
     my ($pred, $l) = @_;
     weaken $_[1];
     $l = force $l;
@@ -971,10 +976,11 @@ sub stream_any ($ $) {
     }
 }
 
-*FP::List::List::stream_any = flip * stream_any;
+*FP::List::List::stream_any = flip \&stream_any;
 
 # (meant as a debugging tool: turn stream to string)
-sub stream_show ($) {
+sub stream_show {
+    @_ == 1 or fp_croak_nargs 1;
     my ($s) = @_;
     join("", map {"  '$_'\n"} @{ stream_to_array $s })
 }
@@ -985,10 +991,10 @@ sub stream_show ($) {
 # the left). Then it's basically a fold_right.
 
 sub stream_state_fold_right {
-    @_ == 3 or die "wrong number of arguments";
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $stateupfn, $s) = @_;
     sub {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($statedown) = @_;
         no warnings 'recursion';
         FORCE $s;
@@ -1003,7 +1009,7 @@ sub stream_state_fold_right {
     }
 }
 
-*FP::List::List::stream_state_fold_right = rot3left * stream_state_fold_right;
+*FP::List::List::stream_state_fold_right = rot3left \&stream_state_fold_right;
 
 TEST {
     stream_state_fold_right(
@@ -1076,17 +1082,17 @@ TEST {
 # introduction?)
 
 sub stream_mixed_fold_right {
-    @_ == 3 or die "wrong number of arguments";
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $state, $v) = @_;
     weaken $_[2] if ref $_[2];
     @_ = ($fn, $state, stream_mixed_flatten $v);
     goto \&stream_fold_right
 }
 
-*FP::List::List::stream_mixed_fold_right = rot3left * stream_mixed_fold_right;
+*FP::List::List::stream_mixed_fold_right = rot3left \&stream_mixed_fold_right;
 
 sub stream_mixed_state_fold_right {
-    @_ == 3 or die "wrong number of arguments";
+    @_ == 3 or fp_croak_nargs 3;
     my ($fn, $statefn, $v) = @_;
     weaken $_[2] if ref $_[2];
     @_ = ($fn, $statefn, stream_mixed_flatten $v);
@@ -1094,12 +1100,12 @@ sub stream_mixed_state_fold_right {
 }
 
 *FP::List::List::stream_mixed_state_fold_right
-    = rot3left * stream_mixed_state_fold_right;
+    = rot3left \&stream_mixed_state_fold_right;
 
 # 'cross product'
 
 sub stream_cartesian_product_2 {
-    @_ == 2 or die "wrong number of arguments";
+    @_ == 2 or fp_croak_nargs 2;
     my ($a, $orig_b) = @_;
     weaken $_[0];
     weaken $_[1];
@@ -1111,9 +1117,9 @@ sub stream_cartesian_product_2 {
             if (is_null $a) {
                 null
             } elsif (is_null $b) {
-                &$rec(cdr $a, $orig_b);
+                &$rec(cdr($a), $orig_b);
             } else {
-                cons(cons(car $a, car $b), &$rec($a, cdr $b))
+                cons(cons(car($a), car($b)), &$rec($a, cdr $b))
             }
         }
     };
@@ -1186,9 +1192,9 @@ list(list(1, 3), list(1, 4), list(2, 3), list(2, 4));
 sub make_chunks_of {
     my ($strictly, $lazy) = @_;
     sub {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($s, $chunklen) = @_;
-        $chunklen >= 1 or die "invalid chunklen: $chunklen";
+        $chunklen >= 1 or croak "invalid chunklen: $chunklen";
         weaken $_[0] if $lazy;    # although tie down is only chunk sized
         require FP::PureArray;
         fix(
@@ -1377,7 +1383,7 @@ TEST { my $s = stream_iota; stream_to_array stream_slice $s, $s }
 [];
 TEST { my $s = stream_iota; stream_to_array stream_slice $s, cdr $s }
 [0];
-TEST { my $s = stream_iota; stream_to_array stream_slice cdr $s, cdddr $s }
+TEST { my $s = stream_iota; stream_to_array stream_slice cdr($s), cdddr $s }
 [1, 2];
 
 # OO interface:
