@@ -94,17 +94,20 @@ use FP::List;
 use Chj::TEST;
 use FP::Combinators qw(flip2of3 flip);
 use Scalar::Util qw(blessed);
+use FP::Carp;
 
 package FP::StrictList::List {
+    use FP::Carp;
+    use Chj::NamespaceCleanAbove;
 
     sub strictlist {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
         $s
     }
 
     sub list {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my $s = shift;
 
         # Should it *really* convert to a non-strict list? This is
@@ -118,7 +121,7 @@ package FP::StrictList::List {
     }
 
     sub stream {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($l) = @_;
 
         # XX isn't this stupid? Same as above. The current load test
@@ -130,11 +133,14 @@ package FP::StrictList::List {
 package FP::StrictList::Null {
     our @ISA = qw(FP::StrictList::List FP::List::Null);
 
+    use FP::Carp;
+    use Chj::NamespaceCleanAbove;
+
     sub pair_namespace {"FP::StrictList::Pair"}
     *null = \&FP::StrictList::strictnull;
 
     sub cons {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my $s = shift;
 
         # different than FP::List::Null::cons in that it needs to set
@@ -156,12 +162,15 @@ package FP::StrictList::Null {
 package FP::StrictList::Pair {
     our @ISA = qw(FP::StrictList::List FP::List::Pair);
 
+    use FP::Carp;
+    use Chj::NamespaceCleanAbove;
+
     *null = \&FP::StrictList::strictnull;
 
     # represented as blessed [ v, pair-or-null, length]
 
     sub cons {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my $s = shift;
         bless [$_[0], $s, $$s[2] + 1], ref $s
     }
@@ -185,6 +194,9 @@ package FP::StrictList::Pair {
 my $null = bless [], "FP::StrictList::Null";
 
 sub strictnull () {
+
+    # @_ == 0 or fp_croak_nargs 0;
+    # no, is also called as a method
     $null
 }
 
@@ -217,7 +229,8 @@ TEST {
 }
 cons(5, cons(6, strictnull));
 
-sub is_strictlist ($) {
+sub is_strictlist {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     my $r = blessed($v) // return;
     (
@@ -280,7 +293,7 @@ TEST {
 
 TEST {
     my $l = strictlist(7, 8, 9)->reverse;
-    [is_strictlist $l, $l->car, $l->length]
+    [is_strictlist($l), $l->car, $l->length]
 }
 [1, 9, 3];
 
@@ -296,8 +309,8 @@ bless([7, 8], 'FP::List::Pair');
 sub make_reverse__map_with_length_with_tail {
     my ($cons) = @_;
 
-    sub ($$$) {
-        @_ == 3 or die "wrong number of arguments";
+    sub {
+        @_ == 3 or fp_croak_nargs 3;
         my ($fn, $l, $tail) = @_;
         my $a;
         while (!$l->is_null) {
@@ -309,7 +322,7 @@ sub make_reverse__map_with_length_with_tail {
     }
 }
 
-sub strictlist_reverse__map_with_length_with_tail ($$$);
+sub strictlist_reverse__map_with_length_with_tail;
 *strictlist_reverse__map_with_length_with_tail
     = make_reverse__map_with_length_with_tail(\&cons);
 
@@ -323,8 +336,8 @@ TEST {
 }
 ['', [[c => 1], [b => 2], [a => 3]]];
 
-sub strictlist_reverse__map_with_length ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub strictlist_reverse__map_with_length {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     strictlist_reverse__map_with_length_with_tail($fn, $l, strictnull)
 }
@@ -338,8 +351,8 @@ TEST {
 }
 [1, [[c => 1], [b => 2], [a => 3]]];
 
-sub strictlist_array__reverse__map_with_length ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub strictlist_array__reverse__map_with_length {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     my $i = $l->length;
     make_reverse__map_with_length_with_tail(
@@ -361,8 +374,8 @@ TEST {
 }
 [[c => 1], [b => 2], [a => 3]];
 
-sub strictlist_array__map_with_length ($$) {
-    @_ == 2 or die "wrong number of arguments";
+sub strictlist_array__map_with_length {
+    @_ == 2 or fp_croak_nargs 2;
     my ($fn, $l) = @_;
     my $i   = 0;
     my $len = $l->length;
