@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2015-2019 Christian Jaeger, copying@christianjaeger.ch
+# Copyright (c) 2015-2020 Christian Jaeger, copying@christianjaeger.ch
 #
 # This is free software, offered under either the same terms as perl 5
 # or the terms of the Artistic License version 2 or the terms of the
@@ -83,6 +83,8 @@ our @EXPORT      = qw(T TC trampoline);
 our @EXPORT_OK   = qw();
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
+use FP::Carp;
+
 sub T (&) {
     bless $_[0], "FP::Trampoline::Continuation"
 }
@@ -91,7 +93,8 @@ sub TC {
     bless [@_], "FP::Trampoline::Call"
 }
 
-sub trampoline ($) {
+sub trampoline {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     @_ = ();    # so that calling a continuation does not need () (possible
                 # speedup)
