@@ -92,6 +92,8 @@ our @EXPORT      = qw(empty_trie);
 our @EXPORT_OK   = qw($empty_trie);                  # ok?
 our %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
 
+use FP::Carp;
+
 package FP::Trie::KeyNotFoundException {
     use FP::Struct [], 'FP::Struct::Show', 'FP::Abstract::Pure';
     use overload '""' => 'string';
@@ -112,13 +114,14 @@ package FP::Trie::BareLevel {
     use FP::List;
     use FP::Stream;
     use Scalar::Util qw(blessed);
+    use FP::Carp;
 
     use FP::Struct ["sublevels"], "FP::Trie::Trie";
 
     sub perhaps_value { () }
 
     sub perhaps_skip {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($t,  $l)        = @_;
         my ($t2, $maybe_l2) = $t->skip($l);
         defined $maybe_l2 ? () : ($t2)
@@ -127,7 +130,7 @@ package FP::Trie::BareLevel {
     our $key_not_found_exception = new FP::Trie::KeyNotFoundException;
 
     sub perhaps_ref {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($t, $l) = @_;
         if (my ($t2) = $t->perhaps_skip($l)) {
             $t2->perhaps_value
@@ -135,10 +138,10 @@ package FP::Trie::BareLevel {
             ()
         }
     }
-    *maybe_ref = perhaps_to_maybe * perhaps_ref;
-    *ref_or    = perhaps_to_or * perhaps_ref;
-    *ref       = perhaps_to_x * perhaps_ref, $key_not_found_exception;
-    *exists    = perhaps_to_exists * perhaps_ref;
+    *maybe_ref = perhaps_to_maybe \&perhaps_ref;
+    *ref_or    = perhaps_to_or \&perhaps_ref;
+    *ref       = perhaps_to_x \&perhaps_ref, $key_not_found_exception;
+    *exists    = perhaps_to_exists \&perhaps_ref;
 
     # returns ($ending_level, $maybe_keyremainder,
     # $maybe_lastvaluelevel, $maybe_keyremainder_lvl)
@@ -170,7 +173,7 @@ package FP::Trie::BareLevel {
     }
 
     sub update {
-        @_ == 3 or die "wrong number of arguments";
+        @_ == 3 or fp_croak_nargs 3;
         my ($t, $l, $fn) = @_;
         if ($l->is_null) {
             FP::Trie::ValueLevel->new($$t{sublevels}, &$fn($t->perhaps_value))
@@ -194,7 +197,7 @@ package FP::Trie::BareLevel {
     }
 
     sub xdelete {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($t, $l) = @_;
         if ($l->is_null) {
             if ((defined blessed $t) && $t->isa("FP::Trie::ValueLevel")) {
@@ -231,7 +234,7 @@ package FP::Trie::BareLevel {
     }
 
     sub delete {
-        @_ == 2 or die "wrong number of arguments";
+        @_ == 2 or fp_croak_nargs 2;
         my ($t, $l) = @_;
 
         # This is really calling for call/cc (parametrize the end case
@@ -259,13 +262,13 @@ package FP::Trie::BareLevel {
     }
 
     sub set {
-        @_ == 3 or die "wrong number of arguments";
+        @_ == 3 or fp_croak_nargs 3;
         my ($t, $l, $v) = @_;
         $t->update($l, sub {$v})
     }
 
     sub sublevels_length {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($t) = @_;
         hash_length $t->sublevels
     }
@@ -298,7 +301,7 @@ package FP::Trie::BareLevel {
     }
 
     sub keys {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($t) = @_;
         $t->alist->stream_map(sub { $_[0][0] })
     }
@@ -307,7 +310,7 @@ package FP::Trie::BareLevel {
     # else should this be named? `valuestream`? And the above
     # `keystream`?
     sub values {
-        @_ == 1 or die "wrong number of arguments";
+        @_ == 1 or fp_croak_nargs 1;
         my ($t) = @_;
         $t->alist->stream_map(sub { $_[0][1] })
     }
