@@ -78,15 +78,18 @@ use FP::Weak 'weaken';    # instead of from Scalar::Util so that it can
                           # be turned off globally (and we depend on FP
                           # anyway)
 use Scalar::Util qw(blessed refaddr);
+use FP::Carp;
 
-sub is_somearray ($) {
+sub is_somearray {
+    @_ == 1 or fp_croak_nargs 1;
     my $r = ref($_[0]);
 
     # XX mess, make this a proper dependency
     $r eq "ARRAY" or $r eq "PXML::Body"
 }
 
-sub is_empty_string ($) {
+sub is_empty_string {
+    @_ == 1 or fp_croak_nargs 1;
     defined $_[0] and !length ref $_[0] and $_[0] eq ""
 }
 
@@ -111,9 +114,10 @@ sub content_escape {
     $str
 }
 
-sub pxmlforce ($);
+sub pxmlforce;
 
-sub pxmlforce ($) {
+sub pxmlforce {
+    @_ == 1 or fp_croak_nargs 1;
     my ($v) = @_;
     if (my $r = ref $v) {
         if ($r eq "CODE") {
@@ -126,9 +130,8 @@ sub pxmlforce ($) {
     }
 }
 
-sub object_force_escape ($$$$);
-
-sub object_force_escape ($$$$) {
+sub object_force_escape {
+    @_ == 4 or fp_croak_nargs 4;
     my ($v, $string_method_for_context, $escape, $fh) = @_;
 
     # $v is certain to be a reference (XX ehr, not even that?!), but
@@ -182,7 +185,7 @@ sub _attribute_val_to_string {
 }
 
 sub _pxml_print_fragment_fast {
-    @_ == 4 or die "wrong number of arguments";
+    @_ == 4 or fp_croak_nargs 4;
     my ($v, $fh, $html5compat, $void_element_h) = @_;
     weaken $_[0]
 
@@ -383,9 +386,8 @@ LP: {
     }
 }
 
-sub pxml_print_fragment_fast ($ $ );
-
-sub pxml_print_fragment_fast ($ $ ) {
+sub pxml_print_fragment_fast {
+    @_ == 2 or fp_croak_nargs 2;
     my ($v, $fh) = @_;
     weaken $_[0] if ref $_[0];    # ref check perhaps unnecessary here
     my $no_element = sub {
@@ -419,7 +421,8 @@ sub pxml_print_fragment_fast ($ $ ) {
     }
 }
 
-sub pxml_xhtml_print_fast ($ $ ;$ ) {
+sub pxml_xhtml_print_fast {
+    @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
     my ($v, $fh, $maybe_lang) = @_;
     weaken $_[0] if ref $_[0];    # ref check perhaps unnecessary here
     if (not ref $v or (defined(blessed $v) and not $v->isa("PXML::Element"))) {
@@ -451,19 +454,21 @@ sub pxml_xhtml_print_fast ($ $ ;$ ) {
 }
 
 # for now,
-sub pxml_xhtml_print ($ $ ;$ );
+sub pxml_xhtml_print;
 *pxml_xhtml_print = *pxml_xhtml_print_fast;
 
 use Chj::xopen "xopen_write";
 
-sub pxml_print ($ $ ) {
+sub pxml_print {
+    @_ == 2 or fp_croak_nargs 2;
     my ($v, $fh) = @_;
     weaken $_[0] if ref $_[0];    # ref check perhaps unnecessary here
     xprintln($fh, q{<?xml version="1.0"?>});
     pxml_print_fragment_fast($v, $fh);
 }
 
-sub putxmlfile ($$) {
+sub putxmlfile {
+    @_ == 2 or fp_croak_nargs 2;
     my ($path, $xml) = @_;
     weaken $_[1] if ref $_[0];    # ref check perhaps unnecessary here
     my $f = xopen_write $path;
@@ -481,7 +486,8 @@ sub PXML::Element::xmlfile {
     putxmlfile($path, $v)
 }
 
-sub puthtmlfile ($$;$) {
+sub puthtmlfile {
+    @_ >= 2 and @_ <= 3 or fp_croak_nargs "2-3";
     my ($path, $v, $maybe_lang) = @_;
     weaken $_[1] if ref $_[0];    # ref check perhaps unnecessary here
                                   #xmkdir_p dirname $path;
