@@ -1,7 +1,9 @@
 use Test::Requires qw(BSD::Resource);
+use FP::Carp;
 import BSD::Resource;
 
-sub MB ($) {
+sub MB {
+    @_ == 1 or fp_croak_nargs 1;
     $_[0] * 1048576
 }
 
@@ -11,7 +13,8 @@ my $RLIMIT_KIND =
     # memory limit it supports. Linux does, too, so just use that one
     RLIMIT_DATA;
 
-sub setlimit_mem_MB ($) {
+sub setlimit_mem_MB {
+    @_ == 1 or fp_croak_nargs 1;
     my ($limit_MB) = @_;
     my $limit = MB $limit_MB;
     setrlimit $RLIMIT_KIND, $limit, $limit or die "setrlimit: $!";
